% Music vs speech recognition in different environments
% This is a MATLAB implementation of the ICLR'15 paper
% X is the feature
% Y is the label
% Z is the domain descriptor
close all;
clear;
clc;
rng('default');
rng(1943);
load music_vs_speech;
Yr = double(sign(Ytr(:,1)-0.5));         % Train labels.
Ys = double(sign(Yts(:,1)-0.5));         % Test labels.
K = floor(size(Xtr,2)/log(size(Xtr,2))); % Heuristic to estimate the # of latent factors.
P = randn(K,size(Xtr,2));
Q = randn(K,size(Ztr,2));

% config - NN
net = network;

net.numInputs = 4;
net.numLayers = 5;

net.inputConnect(1,1) = 1;
net.inputConnect(2,2) = 1;
net.inputConnect(4,3) = 1;
net.inputConnect(5,4) = 1;

net.layerConnect(3,1) = 1;
net.layerConnect(3,2) = 1;
net.layerConnect(4,3) = 1;
net.layerConnect(5,4) = 1;

net.outputConnect(5) = 1;

net.layers{3}.netInputFcn = 'netprod';
net.layers{4}.netInputFcn = 'netprod';

net.inputs{1}.exampleInput = Xtr';
net.inputs{2}.exampleInput = Ztr';
net.inputs{3}.exampleInput = Yr';
net.inputs{4}.exampleInput = ones(1,size(Xtr,1));

net.layers{1}.size = K;
net.layers{2}.size = K;
net.layers{3}.size = K;
net.layers{4}.size = 1;
net.layers{5}.size = 1;

%net.layers{2}.transferFcn = 'poslin';
net.layers{5}.transferFcn = 'poslin';    %hinge loss max(0,1-y*(w'x))

net.IW{1,1} = P;  % P
net.IW{2,2} = Q;  % Q
net.IW{4,3} = -1; % -y * (w'x); here '-1' brings '-y' along with 'y'
net.IW{5,4} = 1;  % since we need 1-y*(w'x), '1' as input is necessary.

net.LW{3,1} = eye(K);
net.LW{3,2} = eye(K);
net.LW{4,3} = ones(1,K);
net.LW{5,4} = 1;

% Y_hat = sim(net,{Xtr';Ztr';Yr';ones(1,size(Xtr,1))})';

net.inputWeights{4,3}.learn = 0;
net.inputWeights{5,4}.learn = 0;

net.layerWeights{3,1}.learn = 0;
net.layerWeights{3,2}.learn = 0;
net.layerWeights{4,3}.learn = 0;
net.layerWeights{5,4}.learn = 0;

net.performFcn = 'mae'; %'mse'; %'sae'  'sae'
net.trainFcn = 'trainscg'; %'trainlm';
net.divideFcn = 'dividerand';
net.divideParam.trainRatio = 1.0;
net.divideParam.valRatio = 0.0;
net.divideParam.testRatio = 0.0;
net.plotFcns = {'plotperform','plottrainstate'};
net.view
net = train(net,{Xtr';Ztr';Yr';ones(1,size(Xtr,1))},zeros(1,size(Xtr,1)));

%% a
P_est = net.IW{1,1};
Q_est = net.IW{2,2};
ACC_train = 1 - mean(double(sign(diag((Xtr*P_est')*(Q_est*Ztr'))))==Yr)
ACC_test = 1 - mean(double(sign(diag((Xts*P_est')*(Q_est*Zts'))))==Ys)